#version 330 core

uniform sampler2D texture0; // physical data
uniform sampler2D texture1; // mask
uniform sampler1D texture2; // colormap

uniform vec2  range;
uniform float logscale;
uniform float alpha;
uniform vec4 scaleFactor;
uniform vec4 offset;
uniform uint channels;
uniform float glFactor;

in vec2 texCoord0;
out vec4 out_Color;

// Clamp the value between [min, max] and normalize to [0, 1]
float saturateToRange(in float value, in vec2 range)
{
    float result = clamp(value, range.x, range.y);
    return (result - range.x) / (range.y - range.x);
}

void main()
{
    float mask = texture(texture1, texCoord0).r;

    if (mask > 0.0)
    {
        float value;

        if (channels == 1u)
        {
            value = ((texture(texture0, texCoord0).r * glFactor) * scaleFactor.r) + offset.r;
        }
        else
        {
            vec2 raw_value = ((texture(texture0, texCoord0).rg * glFactor) * scaleFactor.rg) + offset.rg;
            value = length(raw_value);
        }

        value = clamp(value, range.x, range.y);

        if (logscale > 0.0)
        {
            value = (log(value) - log(range.x)) / (log(range.y) - log(range.x));
        }
        else
        {
            value = saturateToRange(value, vec2(range.x, range.y));	
        }

        out_Color.rgb = texture(texture2, value).rgb;
        out_Color.a = alpha;
    }
    else
    {
        discard;
    }
}
