#version 330 core

uniform sampler2D texture0; // data
uniform sampler2D texture1; // mask

uniform vec3 scaleFactor;
uniform vec3 offset;
uniform float multiplicator;
uniform vec3  rangeMin; // RGB min
uniform vec3  rangeMax; // RGB max
uniform float alpha;
uniform vec2 range;

#define readTexture3(texUnit, coord) ((texture(texUnit, coord).rgb * multiplicator) * scaleFactor) + offset

in vec2 texCoord0;
out vec4 out_Color;

// Clamp the value between [min, max] and normalize to [0, 1]
//vec3 saturateToRange(in vec3 value, in vec3 rangeMin, in vec3 rangeMax)
//{
//    vec3 result = clamp(value, rangeMin, rangeMax);
//    return (result - rangeMin) / (rangeMax - rangeMin);
//}

void main()
{
    float mask = texture(texture1, texCoord0).r;

    if (mask > 0.0)
    {
        vec3 value = readTexture3(texture0, texCoord0);

        vec3 pmin = rangeMin + range.x * (rangeMax - rangeMin);
        vec3 pmax = rangeMin + range.y * (rangeMax - rangeMin);

        vec3 result = clamp(value, rangeMin, rangeMax);
        result = (result - pmin) / (pmax - pmin);
        result = result.bgr;
        out_Color = vec4(result, alpha);

        //value = saturateToRange(value, rangeMin, rangeMax);	

        //value.rgb = value.bgr;

        // apply gamma correction
        //float gamma = 2.2;
        //value = pow(value, vec3(1.0/gamma));
        //out_Color = vec4(value, alpha);

    }
    else
    {
        discard;
    }
}