#version 330 core

layout(location = 0) in float percent;

uniform vec3 p0;
uniform vec3 p1;
uniform mat4 mvp;

vec3 slerp(vec3 p0, vec3 p1, float t)
{
    float dotp = dot(normalize(p0), normalize(p1));

    if ((dotp > 0.9999) || (dotp<-0.9999))
    {
        if (t<=0.5)
            return p0;
        return p1;
    }

    float theta = acos(dotp * 3.14159/180.0);
    return ((p0*sin((1.0-t)*theta) + p1*sin(t*theta)) / sin(theta));
}

vec3 ToVector3D(vec2 geodetic)
{
    float cosLatitude = cos(geodetic.y);

    return normalize(vec3(
        cosLatitude * cos(geodetic.x),
        cosLatitude * sin(geodetic.x),
        sin(geodetic.y)));
}

void main()
{
    //vec2 geo = mix(p0, p1, percent);
    vec3 p = normalize(slerp(p0, p1, percent));

    //vec3 p = ToVector3D(geo);

    gl_Position = mvp * vec4(p, 1.0);
}
