#version 330 core

// Coordinates of the vertex in the texture in xy,
layout(location = 0) in vec2 position; // xy = lon,lat
// coordinates of the neighbour vertex in the texture in zw,
layout(location = 1) in vec2 neighbour;
// Position of the vertex in the current streamline [0, streamlineLength-1]
layout(location = 2) in float localID;

uniform sampler2D texture0; // physical
uniform sampler2D texture1; // mask
uniform vec4 bbox; // lonMin/latMin/lonMax/latMax
uniform float speed;
uniform float headID;
uniform vec4 scaleFactor;
uniform vec4 offset;
uniform float glFactor;
uniform float rangeMax;
uniform vec2 uvTexelStep;
uniform bvec2 seamless;
uniform float bboxXInverted;
uniform float bboxYInverted;


// New position to pass to the fragment shader
out vec2 position0;

const float PI = 3.141592653589793;

void main()
{
    // Check if it's the streamline head or a part of the body
    if (localID == headID)
    {
        // It's the head, compute her new position with the value of the dataset.

        // First compute the texture coordinates of the particle in the dataset
        // with particle latlon and the dataset bounding box.
        vec2 texCoord;

        vec2 particlePos = position;

        for (int i = 0; i < 20; ++i)
        {
            texCoord.x = (particlePos.x - bbox.x) / (bbox.z - bbox.x);
            texCoord.y = (particlePos.y - bbox.w) / (bbox.y - bbox.w);

            if (0.0 > bboxXInverted)
            {
                texCoord.x = 1.0 - texCoord.x;
            }

            if (0.0 > bboxYInverted)
            {
                texCoord.y = 1.0 - texCoord.y;
            }

            // Read the mask
            float mask = texture(texture1, texCoord).x;

            // Check if there are data here.
            if (mask > 0.0)
            {
                // Data is available
                // Read the vector in the dataset
                vec2 dir = ((texture(texture0, texCoord).rg * glFactor) * scaleFactor.rg) + offset.rg;

                float value = length(dir);
                value = min(value, rangeMax) / rangeMax;

                // Compute the new position
                particlePos = particlePos + (normalize(dir) * value) * speed * uvTexelStep;

                if (false == seamless.x)
                {
                    if (   particlePos.x < bbox.x
                        || particlePos.x > bbox.z)
                    {
                        particlePos = position;
                        break;
                    }
                }

                if (false == seamless.y)
                {
                    if (   particlePos.y > bbox.y
                        || particlePos.y < bbox.w)
                    {
                        particlePos = position;
                        break;
                    }
                }
            }
            else
            {
                // No data available here, particle is stopped and wait for her death.
                break;
            }
        }

        // Save the result to pass to the fragment shader
        position0 = particlePos;
    }
    else
    {
        position0 = neighbour;
    }
}
