#version 330 core

layout(location = 0) in vec2 position;
layout(location = 1) in vec2 neighbour;
// Position of the vertex in the current streamline [0, streamlineLength-1]
layout(location = 2) in float localID;

uniform sampler2D texture0; // physical
uniform sampler2D texture1; // mask
uniform sampler2D texture2; // gcp up vector
uniform float speed;
uniform float headID;
uniform vec4 scaleFactor;
uniform vec4 offset;
uniform float glFactor;
uniform vec2 uvTexelStep;
uniform float rangeMax;
uniform vec2 orientation;
uniform bvec2 seamless;

// New position to pass to the fragment shader
out vec2 position0;

void main()
{
    // Check if it's the streamline head or a part of the body
    if (localID == headID)
    {
        // It's the head, compute her new position with the value of the dataset.

        vec2 particlePos = position;

        for (int i = 0; i < 20; ++i)
        {
            // Read the mask
            float mask = texture(texture1, particlePos).x;

            // Check if there are data here.
            if (mask > 0.0)
            {
                // Data is available
                // Read the vector in the dataset
                vec2 dir = ((texture(texture0, particlePos).rg * glFactor) * scaleFactor.rg) + offset.rg;

                float value = length(dir);
                value = min(value, rangeMax) / rangeMax;
                
                vec2 vY = texture(texture2, particlePos).xy;
                vec2 vX = vec2(-vY.y, vY.x);

                dir = normalize(dir.x * (vX * orientation.x) + dir.y * (vY * orientation.y)) * value;

                // Compute the new position
                particlePos = particlePos + dir * uvTexelStep * speed;

                if (seamless.x)
                {
                    if (particlePos.x >= 1.0)
                    {
                        particlePos.x -= 1.0;
                    }
                    else if (particlePos.x < 0.0)
                    {
                        particlePos.x += 1.0;
                    }
                }
                else if (seamless.y)
                {
                    if (particlePos.y >= 1.0)
                    {
                        particlePos.y -= 1.0;
                    }
                    else if (particlePos.y < 0.0)
                    {
                        particlePos.y += 1.0;
                    }
                }
                else
                {
                    if (particlePos.x >= 1.0 || particlePos.y >= 1.0)
                        break;

                    if (particlePos.x <= 0.0 || particlePos.y <= 0.0)
                        break;
                }
            }
            else
            {
                // No data available here, particle is stopped and wait for her death.
                break;
            }
        }

        // Save the result to pass to the fragment shader
        position0 = particlePos;
    }
    else
    {
        position0 = neighbour;
    }
}
