#version 330 core

layout(location = 0) in vec2 position;
// Position of the vertex in the current streamline [0, streamlineLength-1]
layout(location = 1) in float localID;

uniform sampler2D texture0; // physical data
uniform sampler1D texture1; // colormap
uniform vec2 range;
uniform mat4 mvp;
uniform vec4 bbox;
uniform float streamlineLength;
uniform vec4 scaleFactor;
uniform vec4 offset;
uniform float glFactor;
uniform float bboxXInverted;
uniform float bboxYInverted;

out vec4 color0;


vec3 ToVector3D(vec2 geodetic)
{
    float cosLatitude = cos(geodetic.y);

    return normalize(vec3(
        cosLatitude * cos(geodetic.x),
        cosLatitude * sin(geodetic.x),
        sin(geodetic.y)));
}

// Clamp the value between [min, max] and normalize to [0, 1]
float saturateToRange(in float value, in vec2 range)
{
    float result = clamp(value, range.x, range.y);
    return (result - range.x) / (range.y - range.x);
}

void main()
{
    color0.a = localID / streamlineLength;

    vec2 texCoord;
    texCoord.x = (position.x - bbox.x) / (bbox.z - bbox.x);
    texCoord.y = (position.y - bbox.w) / (bbox.y - bbox.w);

	if (0.0 > bboxXInverted)
	{
		texCoord.x = 1.0 - texCoord.x;
	}

	if (0.0 > bboxYInverted)
	{
		texCoord.y = 1.0 - texCoord.y;
	}

    vec2 uv = ((texture(texture0, texCoord).rg * glFactor) * scaleFactor.rg) + offset.rg;
    float value = length(uv);

    value = saturateToRange(value, range);

    color0.rgb = texture(texture1, value).rgb;

    gl_Position = mvp * vec4(ToVector3D(position), 1.0);
}
