#version 330 core

layout(location = 0) in vec2 position;
// Position of the vertex in the current streamline [0, streamlineLength-1]
layout(location = 1) in float localID;

uniform sampler2D texture0; // gcp

uniform mat4 mvp;
uniform float streamlineLength;
uniform vec3 fillColor;
uniform vec2 size;
uniform vec2 texelSize;

out vec4 color0;

vec4 tex2DBiLinear( sampler2D textureSampler_i, vec2 texCoord_i)
{
    // Center of the texel
    vec2 texcoord = vec2((floor(texCoord_i.x * size.x) + 0.5)/ size.x, (floor(texCoord_i.y * size.y) + 0.5) / size.y);

    vec4 p0q0 = texture(textureSampler_i, texcoord);
    vec4 p1q0 = texture(textureSampler_i, texcoord + vec2(texelSize.x, 0));

    vec4 p0q1 = texture(textureSampler_i, texcoord + vec2(0, texelSize.y));
    vec4 p1q1 = texture(textureSampler_i, texcoord + texelSize);

    float a = fract( texCoord_i.x * size.x ); // Get Interpolation factor for X direction.
					// Fraction near to valid data.

    vec4 pInterp_q0 = mix( p0q0, p1q0, a ); // Interpolates top row in X direction.
    vec4 pInterp_q1 = mix( p0q1, p1q1, a ); // Interpolates bottom row in X direction.

    float b = fract( texCoord_i.y * size.y );// Get Interpolation factor for Y direction.

    return mix( pInterp_q0, pInterp_q1, b ); // Interpolate in Y direction.
}

void main()
{
    color0.a = localID / streamlineLength;
    color0.rgb = fillColor;

    vec3 particlePos = tex2DBiLinear(texture0, position).xyz;

    gl_Position = mvp * vec4(particlePos, 1.0);
}
