#version 330 core

layout(location = 0) in vec2 position;
// Position of the vertex in the current streamline [0, streamlineLength-1]
layout(location = 1) in float localID;

uniform sampler2D texture0; // gcp
uniform sampler2D texture1; // physical data
uniform sampler1D texture2; // colormap

uniform mat4 mvp;
uniform float streamlineLength;
uniform vec2 range;
uniform vec4 scaleFactor;
uniform vec4 offset;
uniform float glFactor;
uniform vec2 size;
uniform vec2 texelSize;

out vec4 color0;

// Clamp the value between [min, max] and normalize to [0, 1]
float saturateToRange(in float value, in vec2 range)
{
    float result = clamp(value, range.x, range.y);
    return (result - range.x) / (range.y - range.x);
}

vec4 tex2DBiLinear( sampler2D textureSampler_i, vec2 texCoord_i)
{
    // Center of the texel
    vec2 texcoord = vec2((floor(texCoord_i.x * size.x) + 0.5)/ size.x, (floor(texCoord_i.y * size.y) + 0.5) / size.y);

    vec4 p0q0 = texture(textureSampler_i, texcoord);
    vec4 p1q0 = texture(textureSampler_i, texcoord + vec2(texelSize.x, 0));

    vec4 p0q1 = texture(textureSampler_i, texcoord + vec2(0, texelSize.y));
    vec4 p1q1 = texture(textureSampler_i, texcoord + texelSize);

    float a = fract( texCoord_i.x * size.x ); // Get Interpolation factor for X direction.
					// Fraction near to valid data.

    vec4 pInterp_q0 = mix( p0q0, p1q0, a ); // Interpolates top row in X direction.
    vec4 pInterp_q1 = mix( p0q1, p1q1, a ); // Interpolates bottom row in X direction.

    float b = fract( texCoord_i.y * size.y );// Get Interpolation factor for Y direction.

    return mix( pInterp_q0, pInterp_q1, b ); // Interpolate in Y direction.
}

void main()
{
    vec2 uv = ((texture(texture1, position).rg * glFactor) * scaleFactor.rg) + offset.rg;
    float value = length(uv);

    value = saturateToRange(value, range);

    color0.rgb = texture(texture2, value).rgb;
    color0.a = localID / streamlineLength;

    vec3 particlePos = tex2DBiLinear(texture0, position).xyz;

    gl_Position = mvp * vec4(particlePos, 1.0);
}
