#version 330 core

layout(location = 0) in vec2 direction;

uniform mat4 mvp;
uniform vec2 viewport;
// half diagonal size, square edge = (arrowTileSize * 2) / sqrt(2)
uniform float arrowTileSize; 
uniform vec3 position;

vec2 project(vec4 P)
{
    vec2 p = 0.5 + (P.xyz/P.w).xy * 0.5;
    return p * viewport;
}

vec4 unproject(vec2 p, float z, float w)
{
    vec4 P = vec4( w*((p/viewport)*2.0 - 1.0), z, w);
    return P;
}

void main() 
{
    vec3 pos = position;
    
    vec4 currentProjected = mvp * vec4(pos, 1.0);
    vec2 currentScreen    = project(currentProjected);

    // Compute pixel coordinates in Normalized Device Coordinates space [0, 1].
    vec2 pixelCoord = 0.5 + (currentProjected.xyz/currentProjected.w).xy * 0.5;

    // Compute the position of the current corner vertex in screen space
    vec2 dirNorm = normalize(direction.xy);

    vec2 currentScreen2 = currentScreen + dirNorm * arrowTileSize;

    vec4 p = unproject(currentScreen2, currentProjected.z, currentProjected.w); 

    gl_Position = p;
}
