precision mediump float;

uniform sampler2D texture0; // data
uniform sampler2D texture1; // mask
uniform sampler2D texture2; // palette

uniform float min_value;
uniform float max_value;
uniform float logscale;
uniform vec2 textureRatio;
uniform float alpha;
uniform vec4 uncompress; // multiplicator, offset, scaleFactor, channels

#define MULTIPLICATOR   uncompress.r
#define OFFSET          uncompress.g
#define SCALEFACTOR     uncompress.b
#define CHANNELS        uncompress.a

#define readTexture1(texUnit, coord) ((texture2D(texUnit, coord).r * MULTIPLICATOR) * SCALEFACTOR) + OFFSET
#define readTexture2(texUnit, coord) ((texture2D(texUnit, coord).ra * MULTIPLICATOR) * SCALEFACTOR) + OFFSET

varying vec2 texCoord0;

void main()
{
    vec2 texCoordRatio = texCoord0 * textureRatio;

    float mask = texture2D(texture1, texCoordRatio).r;

    if (mask > 0.0)
    {
        float value;

        if (CHANNELS == 1.0)
        {
            value = readTexture1(texture0, texCoordRatio);
        }
        else
        {
            vec2 raw_value = readTexture2(texture0, texCoordRatio);
            value = length(raw_value);
        }

        value = clamp(value, min_value, max_value);
		if (logscale > 0.0)
	   {
			   value = (log(value) - log(min_value)) / (log(max_value) - log(min_value));
	   }
	   else
	   {
			   value = (value - min_value) / (max_value - min_value);
	   }
	
		gl_FragColor.rgb = texture2D(texture2, vec2(value, 0.5)).rgb;
		gl_FragColor.a = alpha;
    }
    else
    {
       discard;
    }
}
