#version 100

precision highp float;

attribute vec4 position;
attribute float localID;

uniform sampler2D texture0; // 
uniform sampler2D texture1; // 
uniform sampler2D texture2; // 
//uniform sampler2D texture3; // 
uniform vec4 bbox; // lonMin/latMin/lonMax/latMax
uniform vec2 ratio; // lonMin/latMin/lonMax/latMax
uniform float speed;
uniform float headID;
uniform float time;

varying vec4 position0;

const float PI = 3.141592653589793;

void main()
{
    // Get particle informations
    vec4 particleData = texture2D(texture0, position.xy);

    // Check if it's the streamline head or a part of the body
    if (localID == headID)
    {
        // It's the head, compute her new position with the value of the dataset.

        // First compute the texture coordinates of the particle in the dataset 
        // with particle latlon and the dataset bounding box.
        vec2 texCoord;
        texCoord.x = (particleData.x - bbox.x) / (bbox.z - bbox.x);
        texCoord.y = (particleData.y - bbox.y) / (bbox.w - bbox.y);

        // Read the mask
        float mask = texture2D(texture2, texCoord * ratio).x;

        float lifeTime = time - particleData.z;

        // Check if there are data here.
        if (mask > 0.0 && lifeTime < 20.0)
        {
            // Data is available
            // Read the vector in the dataset
            vec2 dir = texture2D(texture1, texCoord * ratio).xw;

            // Compute the new position
            vec2 newPosition = particleData.xy + dir.xy * speed;

            // Save the result to pass to the fragment shader
            position0.xy = newPosition;
            position0.zw = particleData.zw;
        }
        else
        {
            // No data available here, particle is stopped and wait for her death.
            position0 = particleData;
        }
    }
    else
    {
        // It's a part of the streamline body, juste read and Pass to the fragment 
        // shader the neighbour data. (buffer ring algorithm).
        vec4 neighbourData = texture2D(texture0, position.zw);
        vec4 newPosition = vec4(particleData.xy + (neighbourData.xy - particleData.xy) / 15.0, 0.0, 0.0);

        position0 = newPosition;
    }

    // Transform the [0, 1] coordinates in [-1, 1] coordinates.
    vec2 clipPosition = position.xy * 2.0 - 1.0;

    // Save the fragment coordinates
    gl_Position = vec4(clipPosition, 0.0, 1.0);
}
