precision mediump float;

attribute vec3 position;
attribute vec3 previous;
attribute vec3 next;
attribute vec2 direction; 

uniform mat4    mvp;
uniform float   halfThickness;
uniform vec2    viewport;
uniform vec3    color;

varying vec3 color0;

vec2 project(vec4 P)
{
    vec2 p = 0.5 + (P.xyz/P.w).xy * 0.5;
    return p * viewport;
}

// Project from the screen space to the world space
vec4 unproject(vec2 p, float z, float w)
{
    vec4 P = vec4( w*((p/viewport)*2.0 - 1.0), z, w);
    return P;
}

void main() 
{
    color0 = color;

    // vertex 
    vec4 currentProjected    = mvp * vec4(position, 1.0);

    //get 2D screen space with W divide and aspect correction
    vec2 currentScreen    = project(currentProjected);
    vec2 previousScreen   = project(mvp * vec4(previous, 1.0));
    vec2 nextScreen       = project(mvp * vec4(next, 1.0));

    vec2 orientation = vec2(direction.x, direction.y > 0.0 ? 1.0 : -1.0);

    int type = int(abs(direction.y));

    //starting point uses (next - current)
    vec2 dir = vec2(0.0);

    if (type == 1) 
    {
        dir = normalize(nextScreen - currentScreen);
    } 
    //ending point uses (current - previous)
    else if (type == 3) 
    {
        dir = normalize(currentScreen - previousScreen);
    }
    //somewhere in middle, needs a join
    else 
    {
        // Compute segment direction
        if (direction.x < 0.0)
        {
            dir = normalize(nextScreen - currentScreen);
        }
        else
        {
            dir = normalize(currentScreen - previousScreen);
        }
    }

    vec2 normal = ((dir * orientation.x) + (vec2(-dir.y, dir.x) * orientation.y));
    normal *= halfThickness + 1.25;

    gl_Position = unproject(currentScreen + normal, currentProjected.z, currentProjected.w);
}
