#version 100
/*
attribute vec3 position;
attribute vec3 previous;
attribute vec3 next;
attribute vec2 direction; */

attribute vec2 direction; 
attribute vec3 position;
attribute vec3 next;
attribute vec3 previous;
attribute float data; 

uniform mat4  mvp;
uniform float thickness;
uniform vec2  viewport;
uniform vec2  range;

varying vec2 texCoord0;
varying vec3 color0;
varying float segmentLength;
varying float lineWidth;
varying float data0;

vec2 project(vec4 P)
{
    vec2 p = 0.5 + (P.xyz/P.w).xy * 0.5;
    return p * viewport;
}

// Project from the screen space to the world space
vec4 unproject(vec2 p, float z, float w)
{
    vec4 P = vec4( w*((p/viewport)*2.0 - 1.0), z, w);
    return P;
}

void main() 
{
    mat4 projViewModel = mvp;

    // vertex 
    vec4 currentProjected    = projViewModel * vec4(position, 1.0);

    //get 2D screen space with W divide and aspect correction
    vec2 currentScreen    = project(currentProjected);
    vec2 previousScreen   = project(projViewModel * vec4(previous, 1.0));
    vec2 nextScreen       = project(projViewModel * vec4(next, 1.0));

    lineWidth = thickness / 2.0;
    vec2 orientation = vec2(direction.x, direction.y > 0.0 ? 1.0 : -1.0);

    int type = int(abs(direction.y));

    //starting point uses (next - current)
    vec2 dir = vec2(0.0);

    if (type == 1) 
    //if (abs(direction.y) < 1.5) 
    {
        color0 = vec3(1,0,0);
        dir = normalize(nextScreen - currentScreen);
        segmentLength = distance(currentScreen, nextScreen);
    } 
    //ending point uses (current - previous)
    //else if (abs(direction.y) > 2.5) 
    else if (type == 3) 
    {
        color0 = vec3(0,1,0);

        dir = normalize(currentScreen - previousScreen);
        segmentLength = distance(currentScreen, previousScreen);
    }
    //somewhere in middle, needs a join
    else 
    {

        // Compute segment direction
        vec2 dirA = normalize(currentScreen - previousScreen);
        vec2 dirB = normalize(nextScreen - currentScreen);

        color0 = vec3(0,0,1);

        if (direction.x < 0.0)
        {
            dir = dirB;
            segmentLength = distance(currentScreen, nextScreen);
        }
        else
        {
            dir = dirA;
            segmentLength = distance(currentScreen, previousScreen);
        }
    }

    color0 = vec3(1.0);

    vec2 X = dir * orientation.x;
    vec2 Y = vec2(-dir.y, dir.x) * orientation.y;

    vec2 normal = (X + Y);

    float antialias = 1.0;  
    float decal = (1.25) * antialias;
    normal *= lineWidth + decal;

    texCoord0.x = (lineWidth + decal) * orientation.x + (max(0.0, orientation.x) * segmentLength);
    texCoord0.y = (lineWidth + decal) * orientation.y;

    data0 = (clamp(data, range.x, range.y) - range.x) / (range.y - range.x);

    gl_Position = unproject(currentScreen + normal, currentProjected.z, currentProjected.w);
}
